function [xdot] = Generalized_control_scheme(varargin)
%
% Generalized_control_scheme
% Generated: 10/16/06 19:15:58
%
% [xdot] = Generalized_control_scheme() => xdot = initial conditions in column vector
% [xdot] = Generalized_control_scheme('states') => xdot = species names in cell-array
% [xdot] = Generalized_control_scheme('parameters') => xdot = parameter names in cell-array
% [xdot] = Generalized_control_scheme('parametervalues') => xdot = parameter values in column vector
% [xdot] = Generalized_control_scheme(t,x) => xdot = time derivatives in column vector
% [xdot] = Generalized_control_scheme(t,x,PARAMETERS,PARAMETERVALUES) => xdot = time
%       derivatives in column vector. PARAMETERS is a cell-array of strings
%       containing the parameters which will have changed values,
%       PARAMETERVALUES is a vector of parametervalues for the parameters
%       to be changed.
%
% State names and ordering:
%
% x(1): Y
% x(2): G
% x(3): Gmono
% x(4): S
% x(5): RNA
% x(6): T
% x(7): Tmono
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% HANDLE VARIABLE ARGUMENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin == 0,
    % Return initial conditions of the state variables
    xdot = [
        100 ; 10 ; 10 ; 1 ; 1
        1 ; 1
    ];
    return
elseif nargin == 1,
    if strcmp(varargin{1}, 'states'),
        % Return state names in cell-array
        xdot = {
            'Y' ; 'G' ; 'Gmono' ; 'S' ; 'RNA'
            'T' ; 'Tmono'
        };
    elseif strcmp(varargin{1}, 'parameters'),
        xdot = {
            'Y_clearance_kf' ; 'Y_clearance_Km' ; 'Y_production_kf' ; 'Gmono_degradation_kf' ; 'translation_kf'
            'transcription_kc' ; 'transcription_n' ; 'transcription_k0' ; 'transcription_Kd' ; 'RNA_degradation_kf'
            'Tmono_degradation_kf' ; 'T_synthesis_kf' ; 'T_oligomerization_kf' ; 'T_oligomerization_kb' ; 'T_oligomerization_n'
            'T_degradation_kf' ; 'G_oligomerization_kf' ; 'G_oligomerization_kb' ; 'G_oligomerization_n' ; 'G_degradation_kf'
        };
    elseif strcmp(varargin{1}, 'parametervalues'),
        xdot = [
            21000 ; 2000 ; 10000 ; 2 ; 20
            10 ; 2 ; 2 ; 3 ; 3
            10000 ; 100 ; 100 ; 100 ; 3
            0 ; 100 ; 100 ; 1 ; 0
        ];
    else
        error('Wrong input arguments! Please read the help text to the ODE file.');
    end
    return
elseif (nargin == 2) | (nargin == 3),
    t = varargin{1};
    x = varargin{2};
    PARAMETERS = [];
    PARAMETERVALUES = [];
elseif nargin == 4,
    t = varargin{1};
    x = varargin{2};
    PARAMETERS = varargin{3};
    if strcmp(class(PARAMETERS),'char'),
        PARAMETERS = {PARAMETERS};
    end
    PARAMETERVALUES = varargin{4};
    if length(PARAMETERS) ~= length(PARAMETERVALUES),
        error('Not the same number of PARAMETERS and PARAMETERVALUES have been supplied!');
    end
else
    error('Wrong input arguments! Please read the help text to the ODE file.');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STATES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Y = x(1);
G = x(2);
Gmono = x(3);
S = x(4);
RNA = x(5);
T = x(6);
Tmono = x(7);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Y_clearance_kf = 21000;
Y_clearance_Km = 2000;
Y_production_kf = 10000;
Gmono_degradation_kf = 2;
translation_kf = 20;
transcription_kc = 10;
transcription_n = 2;
transcription_k0 = 2;
transcription_Kd = 3;
RNA_degradation_kf = 3;
Tmono_degradation_kf = 10000;
T_synthesis_kf = 100;
T_oligomerization_kf = 100;
T_oligomerization_kb = 100;
T_oligomerization_n = 3;
T_degradation_kf = 0;
G_oligomerization_kf = 100;
G_oligomerization_kb = 100;
G_oligomerization_n = 1;
G_degradation_kf = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ADJUST PARAMETERS IF NEEDED
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for k = 1:length(PARAMETERS),
    eval(sprintf('%s = %f;', PARAMETERS{k},PARAMETERVALUES(k)));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% AUXILIARY VARIABLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cell = 1.0;
Y_clearance = ((Y_clearance_kf * G) * Y) / (Y_clearance_Km + Y);
Y_production = Y_production_kf * S;
Gmono_degradation = Gmono_degradation_kf * Gmono;
translation = translation_kf * RNA;
transcription = (transcription_kc * T ^ transcription_n) / (transcription_Kd ^ transcription_n + T ^ transcription_n) + transcription_k0;
RNA_degradation = RNA_degradation_kf * RNA;
Tmono_degradation = (Tmono_degradation_kf * Tmono) / Y;
T_synthesis = T_synthesis_kf;
T_oligomerization = T_oligomerization_kf * Tmono ^ T_oligomerization_n - T_oligomerization_kb * T;
T_degradation = T_degradation_kf * T;
G_oligomerization = G_oligomerization_kf * Gmono ^ G_oligomerization_n - G_oligomerization_kb * G;
G_degradation = G_degradation_kf * G;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DIFFERENTIAL EQUATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Y_dot = -Y_clearance +Y_production;
G_dot = G_oligomerization -G_degradation;
Gmono_dot = -Gmono_degradation +translation -G_oligomerization;
S_dot = 0;
RNA_dot = transcription -RNA_degradation;
T_dot = T_oligomerization -T_degradation;
Tmono_dot = -Tmono_degradation +T_synthesis -3 * T_oligomerization;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RETURN VALUES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
xdot = [
    Y_dot ; G_dot ; Gmono_dot ; S_dot ; RNA_dot
            T_dot ; Tmono_dot
];
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [x] = piecewisePWL(varargin)
for i=2:2:(nargin)
    if varargin{i}
        x = varargin{i-1};
        return
    end
end
if mod(nargin,2) == 0
    error('No otherwise value');
end
x = varargin{nargin};
return

function [YI] = derinterp1(X,Y,XI)
lx = length(X);
YI = [];
yismatrix = prod(size(Y) > 1);
for i = 1:length(XI),
    ubx = find(X < XI(i));
    if isempty(ubx)
        xi1 = 1;
    else
        xi1 = ubx(end);
        if (xi1 == lx), xi1 = xi1 - 1; end
    end
    xi2 = xi1 + 1;
    if ((xi1 < 0) | (xi2 > lx))
        error('The data abscissae is too small');
    end
    dx = X(xi2) - X(xi1);
    if (yismatrix), 
        dy = Y(xi2,:) - Y(xi1,:);
    else, dy = Y(xi2) - Y(xi1);
    end
    if size(dy,1) > 1,
        YI(i,:) = dy ./ dx;
    else
        YI(i) = dy ./ dx;
    end
end
